var gExpandedCols = {
  init: function() {
    window.removeEventListener("load", gExpandedCols.init, false);

    ftvItem.prototype.getText = function gt(aRow, aColName) {
      const kUnknownSize = "???";

      /**
       * Add a prefix to denote the value is actually a sum of all the subfolders.
       * The prefix is useful as this sum may not always be the exact sum of individual
       * folders if they shown expanded (due to rounding to a unit).
       * E.g. folder1 600bytes -> 1KB, folder2 700bytes -> 1KB
       * summarized at parent folder: 1300bytes -> 1KB
       */
      function addSummarizedPrefix(aValue) {
        if (!sumSubfolders)
          return aValue;
        else
          return extraColsBundle.getFormattedString("summarizedValue", [aValue]);
      }

      /**
       * nsIMsgFolder uses -1 as a magic number to mean "I don't know". In those
       * cases we shouldn't display it to the user.
       */
      function fixNum(aNumber) {
        return aNumber <= 0 ? "" : addSummarizedPrefix(aNumber);
      }

      /**
       * Recursively get the size of specified folder and all its subfolders.
       */
      function getFolderSize(aFolder) {
        let size = 0;
        try {
          size = aFolder.sizeOnDisk;
        } catch(ex) {
          return kUnknownSize;
        }
        if (sumSubfolders && aFolder.hasSubFolders) {
          let subFolders = aFolder.subFolders;
          while (subFolders.hasMoreElements()) {
            let subFolder = subFolders.getNext()
              .QueryInterface(Components.interfaces.nsIMsgFolder);
            let subSize = getFolderSize(subFolder);
            if (subSize == kUnknownSize)
              return subSize;
            else
              size += subSize;
          }
        }
        return size;
      }

      // only show counts / total size of subtree if the pref is set,
      // we are in "All folders" mode and this folder row is not expanded.
      let sumSubfolders = Services.prefs
        .getBoolPref("extensions.extra-cols@jminta_gmail.com.sumSubfolders") &&
        gFolderTreeView.mode == kDefaultMode && this._folder.hasSubFolders &&
        !gFolderTreeView._rowMap[aRow].open;

      let extraColsBundle = document.getElementById("bundle_extraCols");

      switch (aColName) {
        case "folderNameCol":
          // Note: Although this looks like ftvItem.text(), it isn't quite
          // the same, as we are adjusting when we show the unread count.

          let text;
          if (this.useServerNameOnly)
            text = this._folder.server.prettyName;
          else {
            text = this._folder.abbreviatedName;
            if (this.addServerName)
              text = extraColsBundle.getFormattedString(
                "folderWithAccount", [text, this._folder.server.prettyName]);
          }

          // If the unread column is shown, we don't need to add the count
          if (!document.getElementById("folderUnreadCol").hidden)
            return text;

          let unread = this._folder.getNumUnread(sumSubfolders);
          if (unread > 0)
            text = extraColsBundle.getFormattedString("folderWithUnreadMsgs",
                                                      [text, addSummarizedPrefix(unread)]);
          return text;

        case "folderUnreadCol":
          return fixNum(this._folder.getNumUnread(sumSubfolders));

        case "folderTotalCol":
          return fixNum(this._folder.getTotalMessages(sumSubfolders));

        case "folderSizeCol":
          let size = getFolderSize(this._folder);
          if (size == 0)
            return "";
          if (size == kUnknownSize)
            return size;

          let messengerBundle = document.getElementById("bundle_messenger");

          // If size is non-zero try to show it in a unit that fits in 3 digits,
          // but if user specified a fixed unit, use that.
          size = Math.round(size / 1024);
          let units = messengerBundle.getString("kiloByteAbbreviation2");
          if (Services.prefs.getCharPref("extensions.extra-cols@jminta_gmail.com.sizeunits") !="KB" &&
            (size > 999 ||
             Services.prefs.getCharPref("extensions.extra-cols@jminta_gmail.com.sizeunits") =="MB"))
          {
            size = Math.round(size / 1024);
            units = messengerBundle.getString("megaByteAbbreviation2");
          }

          // This needs to be updated if the "%.*f" placeholder string
          // in "*ByteAbbreviation2" in messenger.properties changes.
          return addSummarizedPrefix(units.replace("%.*f", size).replace(" ",""));

        default:
          return "";
      }
    };

    // Now fix the folder-tree to actually ask for this text...
    gFolderTreeView.getCellText = function gct(aRow, aCol) {
      return gFolderTreeView._rowMap[aRow].getText(aRow, aCol.id);
    };

    // Only show attributes on the first col
    gFolderTreeView.storedEfcGetCellProperties = gFolderTreeView.getCellProperties;
    gFolderTreeView.getCellProperties = function gcp(aRow, aCol) {
      if (aCol.id != "folderUnreadCol" &&
          aCol.id != "folderTotalCol" &&
          aCol.id != "folderSizeCol")
        return gFolderTreeView.storedEfcGetCellProperties(aRow, aCol);

      return "";
    };

    gFolderTreeView._treeElement.setAttribute("hidecolumnpicker", "false");
    document.getElementById("folderNameCol").setAttribute("hideheader", "false");
  }
};

window.addEventListener("load", gExpandedCols.init, false);
